/*
OpenIO SDS cluster
Copyright (C) 2014 Worldine, original work as part of Redcurrant
Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef OIO_SDS__cluster__conscience__conscience_broken_holder_h
# define OIO_SDS__cluster__conscience__conscience_broken_holder_h 1

/**
 * @addtogroup gridcluster_backend
 * @{
 */

# ifndef BRKM1_PREFIX
#  define BRKM1_PREFIX "META1:"
# endif
# ifndef BRKM1_PATTERN
#  define BRKM1_PATTERN "META1:([^:]+):([^:]+)"
# endif
# ifndef BRKM2_PATTERN
#  define BRKM2_PATTERN "([^:]*):([^:]*):::"
# endif
# ifndef BRKCONTAINER_PATTERN_RM
#  define BRKCONTAINER_PATTERN_RM "([^:]*):([^:]*):([^:]*)"
# endif
# ifndef BRKCONTAINER_PATTERN_PUT
#  define BRKCONTAINER_PATTERN_PUT "([^:]*):([^:]*):([^:]*):([^:]*):([^:]*)"
# endif
# ifndef BRKCONTAINER_PATTERN_FIX
#  define BRKCONTAINER_PATTERN_FIX "([^:]*):([^:]*):([^:]*):([^:]*):([^:]*)"
# endif
# include <metautils/lib/metautils.h>
# include <cluster/conscience/conscience.h>

/**
 *
 */
struct broken_holder_s {
	struct conscience_s *conscience;
	GHashTable *ht_meta2;/**< Maps (addr_info_t*) to (broken_meta2_t*)*/
	GHashTable *ht_meta1;/**< Maps (addr_info_t*) to (broken_meta1_t*)*/
};

/**
 *
 */
typedef struct broken_content_s {
	time_t date_insertion;
	gint   counter;
	gchar  container_id[STRLEN_CONTAINERID];
	gchar  content_path[LIMIT_LENGTH_CONTENTPATH+1];
	gchar  cause[128+1];
} broken_content_t;

/**
 *
 */
typedef struct broken_meta2_s {
	time_t       date_insertion;
	gint         counter;
	addr_info_t  addr;
	gboolean     totally_broken;
	time_t       last_alert_stamp;
	GHashTable  *broken_containers;/**<Maps (gchar*) to (broken_content_t*)*/
} broken_meta2_t;

/**
 *
 */
typedef struct broken_meta1_s {
	time_t       date_insertion;
	gint64       counter;
	addr_info_t  addr;
	gchar        string [ sizeof(BRKM1_PREFIX) + STRLEN_ADDRINFO + 1 ];
} broken_meta1_t;

/* ------------------------------------------------------------------------- */

/**
 * @param conscience
 * @return
 */
struct broken_holder_s* conscience_create_broken_holder ( struct conscience_s *conscience );

/**
 * @param bh
 */
void conscience_destroy_broken_holder( struct broken_holder_s *bh );

/**
 * @param bh
 * @param err
 * @param element
 * @return
 */
void broken_holder_remove_element( struct broken_holder_s *bh,
	const gchar *element );

/**
 * @param bh
 * @param err
 * @param element
 * @return
 */
void broken_holder_add_element( struct broken_holder_s *bh,
	const gchar *element );

/**
 * @param bh
 */
void broken_holder_flush( struct broken_holder_s *bh );

/**
 * @param u
 * @param bm1
 */
typedef gboolean (*on_brk_meta1_f) (gpointer u, struct broken_meta1_s *bm1);

/**
 * @param u
 * @param bm2
 * @param bc
 */
typedef gboolean (*on_brk_content_f) (gpointer u, struct broken_meta2_s *bm2,
	struct broken_content_s *bc);

/**
 * @param bh
 * @param oldest
 * @param udata
 * @param m1
 * @param c
 * @return
 */
gboolean broken_holder_run_elements( struct broken_holder_s *bh, time_t oldest,
	gpointer udata, on_brk_meta1_f m1, on_brk_content_f c);

/**
 * @param m1
 * @return 
 */
gchar* broken_holder_write_meta1( struct broken_meta1_s *m1 );

/**
 * @param m22
 * @return 
 */
gchar* broken_holder_write_meta2( struct broken_meta2_s *m2 );

/**
 * @param m2
 * @param c
 * @return 
 */
gchar* broken_holder_write_content( struct broken_meta2_s *m2, struct broken_content_s *c );

/**
 * @param bh
 * @param element
 */
void broken_holder_fix_element(struct broken_holder_s *bh, const gchar *element);

/**
 * @param bh
 * @param element
 * @return
 */
gboolean broken_holder_check_element_format( struct broken_holder_s *bh, const gchar *element );

/** @} */

#endif /*OIO_SDS__cluster__conscience__conscience_broken_holder_h*/